@echo off

rem getting input parameters
setlocal EnableDelayedExpansion

set TEMP_FILE=%1
set DOMAIN=%~4
set LDAP_CONFIG=%~3
set InstanceName=%5
set PROPS_FILE=%6

set ACTUATOR_LOG="..\log\%PROPS_FILE%.log"

rmdir %PROPS_FILE%.dir /S /q
mkdir %PROPS_FILE%.dir

rem checking parameters
if %TEMP_FILE%=="" (
 echo %date% %time% Output file is not defined >> %ACTUATOR_LOG%
 exit /b 1
)

if %DOMAIN%=="" (
 echo %date% %time% Domain is not defined >> %ACTUATOR_LOG%
 exit /b 1
)

if "%LDAP_CONFIG%"=="" (
 echo %date% %time% LDAP config path is not defined >> %ACTUATOR_LOG%
 exit /b 1
)

if %InstanceName%=="" (
 echo %date% %time% Instance name is not defined >> %ACTUATOR_LOG%
 exit /b 1
)

if %PROPS_FILE%=="" (
 echo %date% %time% PROPS_FILE is not defined >> %ACTUATOR_LOG%
 exit /b 1
)

rem setting LDAP parameters
set DBM_CFG="%PROPS_FILE%.dir\db2_cfg.txt"
set /A LDAP_ENABLED=0
set LDAP_HOST=127.0.0.1
set LDAP_PORT=389
set USER_OBJECTCLASS=inetOrgPerson
set GROUP_OBJECTCLASS=groupOfNames
set USER_BASEDN=c=*
set GROUP_BASEDN=c=*
set USERID_ATTRIBUTE=uid
set AUTHID_ATTRIBUTE=uid
set GROUPNAME_ATTRIBUTE=cn
set SEARCH_DN=
set SEARCH_PW=none
set GROUP_LOOKUP_METHOD=SEARCH_BY_DN
set GROUP_LOOKUP_ATTRIBUTE=member

rem setting role's variables
set ROLE_LIST_TMP="%PROPS_FILE%.dir\role_list_tmp.txt"
set REQUEST_FILE="%PROPS_FILE%.dir\request.txt"
set ROLE_LIST="%PROPS_FILE%.dir\roles.txt"
set USER_LIST="%PROPS_FILE%.dir\users.txt"
set USER_LIST_TMP="%PROPS_FILE%.dir\users.tmp"
set ROLES_TABLE_START="%PROPS_FILE%.dir\roles_table_start.txt"
set ROLES_TABLE_CURRENT="%PROPS_FILE%.dir\roles_table_current.txt"
set ROLES_TABLE_END="%PROPS_FILE%.dir\roles_table_end.txt"
set REPLACES="%PROPS_FILE%.dir\replaces.txt"
set TEMPFILE="%PROPS_FILE%.dir\tempfile.txt"
set TEMPFILE2="%PROPS_FILE%.dir\tempfile2.txt"
set OUTPUT_FILE="%PROPS_FILE%.dir\roles_grouping.txt"
set LDAP_OUTPUT_FILE="%PROPS_FILE%.dir\ldap_output_file.txt"


set DATABASE_LIST_TMP="%PROPS_FILE%.dir\database_list_tmp.txt"
rem the same - final list
set DATABASE_LIST="%PROPS_FILE%.dir\database_list.txt"
set /A GET_ROLES=0
set /A GET_USERS=0

echo %date% %time% Processing started. > %ACTUATOR_LOG%
rem checking if LDAP is enabled
if exist %DBM_CFG% ( del %DBM_CFG% )
db2cmd.exe -c -w "db2 -r %DBM_CFG% get dbm cfg"
if %ERRORLEVEL% NEQ 0 (
 echo %date% %time% Unable to call db2 and get DBM configuration.>> %ACTUATOR_LOG%
 )
for /F "delims== tokens=1,2" %%i in ('type %DBM_CFG%') do (
     if "%%i"==" Client Userid-Password Plugin          (CLNT_PW_PLUGIN) " (
         if "%%j"==" IBMLDAPauthclient" (set /A LDAP_ENABLED=1)
        )
     if "%%i"==" Server Userid-Password Plugin        (SRVCON_PW_PLUGIN) " (
         if "%%j"==" IBMLDAPauthserver" (set /A LDAP_ENABLED=1)
        )
    )
    

rem parsing LDAP_CONFIG for actual values

if %LDAP_ENABLED% EQU 1 (
     if exist "%LDAP_CONFIG%" (
         echo %date% %time% Collecting LDAP users and groups. >> %ACTUATOR_LOG%
         for /F "eol=; delims== tokens=1,*" %%i in ('type "%LDAP_CONFIG%"') do (
             rem echo -"%%i"- -"%%j"-
             for /F %%x in ("%%i") do ( set A=%%~x)
             for /F %%x in ("%%j") do ( set B=%%~x)
             if "!A!"=="LDAP_HOST" set LDAP_HOST=!B!
             if "!A!"=="USER_OBJECTCLASS" set USER_OBJECTCLASS=!B!
             if "!A!"=="GROUP_OBJECTCLASS" set GROUP_OBJECTCLASS=!B!
             if "!A!"=="AUTHID_ATTRIBUTE" set AUTHID_ATTRIBUTE=!B!
             if "!A!"=="GROUPNAME_ATTRIBUTE" set GROUPNAME_ATTRIBUTE=!B!
             if "!A!"=="GROUP_LOOKUP_METHOD" set GROUP_LOOKUP_METHOD=!B!
             if "!A!"=="GROUP_LOOKUP_ATTRIBUTE" set GROUP_LOOKUP_ATTRIBUTE=!B!
             if "!A!"=="SEARCH_PW" set SEARCH_PW=!B!
             if "!A!"=="USER_BASEDN" set USER_BASEDN=%%j
             if "!A!"=="GROUP_BASEDN" set GROUP_BASEDN=%%j
             if "!A!"=="SEARCH_DN" set SEARCH_DN=%%j
             
             for /F "delims=: tokens=1,2" %%x in ("!LDAP_HOST!") do (
                 if %%y GTR . ( 
                     set LDAP_PORT=%%y
                     set LDAP_HOST=%%x
                    )                        
                )
     
            )
            
         java -classpath "..\bin\getViper2ldapgroups.jar" nl.consul.cea.gensub.db2viper.java.CollectViperGroups -LDAPHost?"!LDAP_HOST!" -LDAPPort?"!LDAP_PORT!" -UserName?"!SEARCH_DN!" -Password?"!SEARCH_PW!" -USER_BASEDN?"!USER_BASEDN!" -AUTHID_ATTRIBUTE?"!AUTHID_ATTRIBUTE!" -USER_OBJECTCLASS?"!USER_OBJECTCLASS!" -GROUP_OBJECTCLASS?"!GROUP_OBJECTCLASS!" -GROUP_BASEDN?"!GROUP_BASEDN!" -GROUPNAME_ATTRIBUTE?"!GROUPNAME_ATTRIBUTE!" -GROUP_LOOKUP_METHOD?"!GROUP_LOOKUP_METHOD!" -GROUP_LOOKUP_ATTRIBUTE?"!GROUP_LOOKUP_ATTRIBUTE!" -Filename?"%LDAP_OUTPUT_FILE%" 
         if %ERRORLEVEL% NEQ 0 (
             echo %date% %time% Unable to collect LDAP users.>> %ACTUATOR_LOG%
            ) else ( echo %date% %time% LDAP information is collected successfully.>> %ACTUATOR_LOG% )
        )
    )
    
for /F "tokens=* delims= " %%i in ("%USER_BASEDN%") do ( set USER_BASEDN=%%i )
for /F "tokens=* delims= " %%i in ("%GROUP_BASEDN%") do ( set GRPOUP_BASEDN=%%i )
for /F "tokens=* delims= " %%i in ("%SEARCH_DN%") do ( set SEARCH_DN=%%i )

rem call ldapsearcher
if %LDAP_ENABLED% EQU 1 (
     echo Calling ldapsearcher.jar
    ) else (
         echo %date% %time% Collecting OS and domain users. >> %ACTUATOR_LOG%
         call ..\bin\getgroupsAD.bat %TEMP_FILE% "" "" "" "" %DOMAIN% "" "" "" "" "" "" %PROPS_FILE%
         if %ERRORLEVEL% NEQ 0 (
             echo %date% %time% Unable to collect domain and OS groups and users.>> %ACTUATOR_LOG%
            ) else ( echo %date% %time% Domain and OS information is collected successfully.>> %ACTUATOR_LOG% )
        )

rem collecting roles
echo ^[SourceGroup^]> %OUTPUT_FILE% 
echo %date% %time% Start collecting roles.>> %ACTUATOR_LOG%

rem getting list of the databases
db2cmd.exe -c -w "db2 -r %DATABASE_LIST_TMP% list database directory"
if %ERRORLEVEL% NEQ 0 (
 echo %date% %time% Unable to call db2. Exiting.>> %ACTUATOR_LOG%
 exit /b 1
 )
for /F "tokens=1,2,3,4," %%i in ('type %DATABASE_LIST_TMP%') do (
     if "%%i %%j"=="Database name" (echo %%l>> %DATABASE_LIST%)
    )
	
rem getting roles for each database
for /F %%c in ('type %DATABASE_LIST%') do (
set DBNAME=%%c
echo %date% %time% Working for database !DBNAME!.>> %ACTUATOR_LOG%

rem getting users
type nul > %TEMPFILE%
echo connect to !DBNAME! > %REQUEST_FILE%
echo select grantee,'$',rolename from sysibm.sysroleauth where granteetype = 'U' >> %REQUEST_FILE%
echo terminate >> %REQUEST_FILE%
db2cmd.exe -c -w "db2 -f %REQUEST_FILE% -r %USER_LIST_TMP%"
if %ERRORLEVEL% NEQ 0 (
 echo %date% %time% Unable to call db2.>> %ACTUATOR_LOG%
 ) else ( echo %date% %time% Roles are collected successfully.>> %ACTUATOR_LOG% )
 
findstr $ %USER_LIST_TMP% > %USER_LIST%

for /F "delims=$ tokens=1,2" %%i in ('type %USER_LIST%') do (
     for /F "tokens=1 delims= " %%x in ("%%i") do (
	     set t=%%x
	     rem echo %%x >> %INDEX%
	 	)
     for /F "tokens=1 delims= " %%k in ("%%j") do (
	     echo !t!,%%k>> %TEMPFILE%
		)
 	)
type nul > %USER_LIST%
for /F "tokens=1 delims= " %%i in ('type %TEMPFILE%') do (echo %%i>> %USER_LIST%)

type %USER_LIST% > %TEMPFILE%

rem preparing grouping file from plain %TEMPFILE%
    if %LDAP_ENABLED% EQU 1 (
        for /F "delims=, tokens=1,2" %%i in ('type %TEMPFILE%') do (
         echo "!DBNAME!\%%j">> %OUTPUT_FILE%
    	 echo {>> %OUTPUT_FILE%
    	 echo  match logonname "%%i">> %OUTPUT_FILE%
    	 echo  match instancename "%InstanceName%">> %OUTPUT_FILE%
    	 echo }>> %OUTPUT_FILE%
        )
    )    
    )
        else
    (
        for /F "delims=, tokens=1,2" %%i in ('type %TEMPFILE%') do (
         echo "!DBNAME!\%%j">> %OUTPUT_FILE%
    	 echo {>> %OUTPUT_FILE%
    	 echo  match logonname "%DOMAIN%\%%i">> %OUTPUT_FILE%
    	 echo  match instancename "%InstanceName%">> %OUTPUT_FILE%
    	 echo }>> %OUTPUT_FILE%
        )
    )
)

rem grouping file is ready here

echo %date% %time% Finished collecting roles.>> %ACTUATOR_LOG%

type %OUTPUT_FILE% >> %TEMP_FILE%
type %LDAP_OUTPUT_FILE% >> %TEMP_FILE%

echo %date% %time% Collect of user and group information is successful. >> %ACTUATOR_LOG%

rmdir %PROPS_FILE%.dir /S /q
echo %date% %time% Processing finished. >> %ACTUATOR_LOG%
endlocal