#!/bin/ksh

ACTUATOR_LOG=../log/$5.log

#checking input parameters
if [ "a$1" = "a" ] ; then
 echo `date "+%a %D %T"` Output file name is not defined. >> $ACTUATOR_LOG
 exit 1
else
 TEMPFILE1=$1
fi

if [ "a$2" != "a" ] ; then 
 LDAP_SEQURITY_FILE=$3
else
 echo `date "+%a %D %T"` Path to IBMLDAPSecurity.ini is not defined. >> $ACTUATOR_LOG 
 exit 1 
fi


if [ "a$3" != "a" ] ; then 
 INSTANCE_NAME=$4
else
 echo `date "+%a %D %T"` Instance name is not defined. >> $ACTUATOR_LOG 
 exit 1 
fi

if [ "a$4" = "a" ] ; then 
 echo `date "+%a %D %T"` PROPSFILE is not defined >> $ACTUATOR_LOG 
 exit 1
else
 PROPSFILE=$5
fi

# checking who runs the script
RUN_AS_ROOT=0
ID=`id | sed -e 's/uid=//; s/(.*//'`
#echo $ID
if [ $ID -eq 0 ] ; then
  RUN_AS_ROOT=1
else
  DB2_PATH=`cat /etc/passwd | grep "^$INSTANCE_NAME:" | sed -e "s/^$INSTANCE_NAME:[^:]*:[^:]*:[^:]*:[^:]*:\([^:]*\):[^:]*/\1/"`/sqllib/bin
  if [ ! $? -eq 0 ] ; then
   echo `date "+%a %D %T"` Error reading /etc/passwd file. Exiting. >> $ACTUATOR_LOG
   return 1
  fi
fi

# finding java
case `uname` in 
  "AIX")
      JAVA_PATH=`cat /etc/passwd | grep "^$INSTANCE_NAME:" | sed -e "s/^$INSTANCE_NAME:[^:]*:[^:]*:[^:]*:[^:]*:\([^:]*\):[^:]*/\1/"`/sqllib/java/jdk64/bin
     ;;
  "SunOS")
      JAVA_PATH=`cat /etc/passwd | grep "^$INSTANCE_NAME:" | sed -e "s/^$INSTANCE_NAME:[^:]*:[^:]*:[^:]*:[^:]*:\([^:]*\):[^:]*/\1/"`/sqllib/java/jdk64/bin
     ;;
  "Linux")
     JAVA_PATH=`cat /etc/passwd | grep "^$INSTANCE_NAME:" | sed -e "s/^$INSTANCE_NAME:[^:]*:[^:]*:[^:]*:[^:]*:\([^:]*\):[^:]*/\1/"`/sqllib/java/jdk32/bin
     ;;
  *) 
     JAVA_PATH=`cat /etc/passwd | grep "^$INSTANCE_NAME:" | sed -e "s/^$INSTANCE_NAME:[^:]*:[^:]*:[^:]*:[^:]*:\([^:]*\):[^:]*/\1/"`/sqllib/java/jdk32/bin
     ;;
esac

# internal variables
if [ $RUN_AS_ROOT -eq 1 ] ; then
 CURRENT_DIR=`su - $INSTANCE_NAME -c "pwd" | tail -1`
else
 CURRENT_DIR=`pwd`
fi
TEMP_DIR="$CURRENT_DIR/$PROPSFILE.dir"
TMPFILE=$TEMP_DIR/file.txt
DATABASE_LIST_TMP=$TEMP_DIR/database_list_tmp.txt
DATABASE_LIST=$TEMP_DIR/database_list.txt
REQUEST_FILE=$TEMP_DIR/request.sql
USER_LIST_TMP=$TEMP_DIR/user_list_tmp.txt
USER_LIST=$TEMP_DIR/users.txt
OUTPUT_FILE=$TEMP_DIR/output.txt
LOCAL_OUTPUT_FILE=$TEMP_DIR/local_output.txt

# LDAP variables
LDAP_ENABLED=0
LDAP_HOST=127.0.0.1
LDAP_PORT=389
USER_OBJECTCLASS=inetOrgPerson
GROUP_OBJECTCLASS=groupOfNames
USER_BASEDN="c=*"
GROUP_BASEDN="c=*"
USERID_ATTRIBUTE=uid
AUTHID_ATTRIBUTE=uid
GROUPNAME_ATTRIBUTE=cn
SEARCH_DN=
SEARCH_PW=none
GROUP_LOOKUP_METHOD=SEARCH_BY_DN
GROUP_LOOKUP_ATTRIBUTE=member

echo `date "+%a %D %T"` Processing started. >> $ACTUATOR_LOG
echo `date "+%a %D %T"` Start getting roles. >> $ACTUATOR_LOG

#cleaning output file
if [ -f $TEMPFILE1 ] ; then
 rm -f $TEMPFILE1 
fi

#creating temporary directory
rm -rf $TEMP_DIR
if [ $RUN_AS_ROOT -eq 1 ] ; then
  su - $INSTANCE_NAME -c "mkdir $TEMP_DIR "
else
  mkdir $TEMP_DIR
fi

# getting db2 settings
if [ RUN_AS_ROOT -eq 0 ] ; then
 $DB2_PATH/db2 get dbm cfg > $TMPFILE
 if [ ! $? -eq 0 ] ; then
  echo `date "+%a %D %T"` db2 run error. Exiting. >> $ACTUATOR_LOG
  return 0
 fi
else
 su - "$INSTANCE_NAME" -c "db2 get dbm cfg" > $TMPFILE
 if [ ! $? -eq 0 ] ; then
  echo `date "+%a %D %T"` db2 run error. Exiting. >> $ACTUATOR_LOG
 return 0
 fi
fi

#checking if LDAP is enable
TMP=`cat $TMPFILE | grep "Server Userid-Password Plugin.*= IBMLDAPauthserver$" | tail -1`
if [ "a$TMP" != "a" ] ; then
 LDAP_ENABLED=1
fi
TMP=`cat $TMPFILE | grep "Server Userid-Password Plugin.*= IBMLDAPauthclient$" | tail -1`
if [ "a$TMP" != "a" ] ; then
 LDAP_ENABLED=1
fi

if [ $LDAP_ENABLED=0 ] ; then
 echo "[SourceGroup]" > $OUTPUT_FILE
fi

# getting list of the databases
if [ RUN_AS_ROOT -eq 0 ] ; then
 $DB2_PATH/db2 list database directory > $DATABASE_LIST_TMP
else
 su - "$INSTANCE_NAME" -c "db2 list database directory" > $DATABASE_LIST_TMP
fi
#getting list of databases
cat $DATABASE_LIST_TMP | grep "Database name"  | cut -d = -f 2 | sed -e 's/^ //'> $DATABASE_LIST

for i in `cat $DATABASE_LIST`; do
  echo `date "+%a %D %T"` Working for database $i. >> $ACTUATOR_LOG
  # getting users
  # creating request file
  echo "connect to $i" > $REQUEST_FILE
  echo "select grantee,'$',rolename from sysibm.sysroleauth where granteetype = 'U'" >> $REQUEST_FILE
  echo "terminate" >> $REQUEST_FILE
  # creating list of the users 
  if [ $RUN_AS_ROOT -eq 1 ] ; then
   su - $INSTANCE_NAME -c "db2 -f $REQUEST_FILE" > $USER_LIST_TMP
   if [ ! $? -eq 0 ] ; then
      echo `date "+%a %D %T"` Failed to collect roles.>> $ACTUATOR_LOG
   else
      echo `date "+%a %D %T"` Roles are collected successfully. >> $ACTUATOR_LOG
   fi
  else
   $DB2_PATH/db2 -f $REQUEST_FILE > $USER_LIST_TMP
   if [ ! $? -eq 0 ] ; then
      echo `date "+%a %D %T"` Failed to collect roles.>> $ACTUATOR_LOG
   else
      echo `date "+%a %D %T"` Roles are collected successfully. >> $ACTUATOR_LOG
   fi
  fi
  cat $USER_LIST_TMP | grep "\$ " | sed -e 's/ //g;s/\$/,/g' > $USER_LIST 
  for x in `cat $USER_LIST`; do
    ROLE_FOUND=0
    a=`echo $x | cut -d , -f 1`
    b=`echo $x | cut -d , -f 2`
    echo "\"$i\\$b"\" >> $OUTPUT_FILE
    echo { >> $OUTPUT_FILE
    echo "     match rolename \"$a"\" >> $OUTPUT_FILE
    echo "     match instancename \"$INSTANCE_NAME"\" >> $OUTPUT_FILE
    echo } >> $OUTPUT_FILE    
  done
done
echo `date "+%a %D %T"` Finished getting roles. >> $ACTUATOR_LOG

# begin to work with LDAP
if [ -f $LDAP_SEQURITY_FILE ] ; then
  if [ $LDAP_ENABLED -eq 1 ] ; then
   echo `date "+%a %D %T"` Start getting LDAP users and groups. >> $ACTUATOR_LOG
   for i in `cat $LDAP_SEQURITY_FILE | grep "^[a-zA-Z0-9]" | sed -e 's/ //g'`; do
    case `echo $i | cut -d = -f 1` in
     "LDAP_HOST")
                 LDAP_HOST=`echo $i | cut -d = -f 2` 
                 ;;
     "USER_OBJECTCLASS")
                 USER_OBJECTCLASS=`echo $i | cut -d = -f 2`
                 ;;
     "GROUP_OBJECTCLASS")
                 GROUP_OBJECTCLASS=`echo $i | cut -d = -f 2`
                 ;;
     "USER_BASEDN")
                 USER_BASEDN=`echo $i | sed -e 's/^USER_BASEDN=//'`
                 ;;
     "GROUP_BASEDN")
                 GROUP_BASEDN=`echo $i | sed -e 's/^GROUP_BASEDN=//'`
                 ;;
     "USERID_ATTRIBUTE")
                 USERID_ATTRIBUTE=`echo $i | cut -d = -f 2`
                 ;;
     "AUTHID_ATTRIBUTE")
                 AUTHID_ATTRIBUTE=`echo $i | cut -d = -f 2`
                 ;;
     "GROUPNAME_ATTRIBUTE")
                 GROUPNAME_ATTRIBUTE=`echo $i | cut -d = -f 2`
                 ;;
     "GROUP_LOOKUP_ATTRIBUTE")
                 GROUP_LOOKUP_ATTRIBUTE=`echo $i | cut -d = -f 2`
                 ;;
     "GROUP_LOOKUP_METHOD")
                GROUP_LOOKUP_METHOD=`echo $i | cut -d = -f 2`
                 ;;
     "SEARCH_DN")
                 SEARCH_DN=`echo $i | sed -e 's/^SEARCH_DN= *//'`
                 ;;
     "SEARCH_PW")
                 SEARCH_PW=`echo $i | cut -d = -f 2`
                 ;;
     *)
                 echo other parameter is defined 
                 ;;
    esac
   done 
   A=$LDAP_HOST
   LDAP_HOST=`echo $A | sed -e 's/:.*$//'`
   LDAP_PORT=`echo $A | sed -e 's/^.*://'`
   if [ "$LDAP_PORT" = "$LDAP_HOST" ] ; then
    LDAP_PORT=389
   fi
  fi
fi

if [ $LDAP_ENABLED -eq 0 ] ; then 
 echo "LDAP is disabled"  
 if [ ! -n "`uname | grep Linux`" ] ; then
   ../bin/getgroups.sh $LOCAL_OUTPUT_FILE
    if [ ! $? -eq 0 ] ; then
      echo `date "+%a %D %T"` Failed to collect OS users and groups information.>> $ACTUATOR_LOG
    else
      echo `date "+%a %D %T"` OS users and groups are collected successfully. >> $ACTUATOR_LOG
    fi
 fi
else 

 $JAVA_PATH/java  -classpath "../bin/getViper2ldapgroups.jar" nl.consul.cea.gensub.db2viper.java.CollectViperGroups -LDAPHost?"$LDAP_HOST" -LDAPPort?"$LDAP_PORT" -UserName?"$SEARCH_DN" -Password?"$SEARCH_PW" -USER_BASEDN?"$USER_BASEDN" -AUTHID_ATTRIBUTE?"$AUTHID_ATTRIBUTE" -USER_OBJECTCLASS?"$USER_OBJECTCLASS" -GROUP_OBJECTCLASS?"$GROUP_OBJECTCLASS" -GROUP_BASEDN?"$GROUP_BASEDN" -GROUPNAME_ATTRIBUTE?"$GROUPNAME_ATTRIBUTE" -GROUP_LOOKUP_METHOD?"$GROUP_LOOKUP_METHOD" -GROUP_LOOKUP_ATTRIBUTE?"$GROUP_LOOKUP_ATTRIBUTE" -Filename?"$TEMPFILE1" 2>> $ACTUATOR_LOG
 if [ ! $? -eq 0 ] ; then
  echo `date "+%a %D %T"` java run error. Failed to collect LDAP information.>> $ACTUATOR_LOG
 else
  echo `date "+%a %D %T"` LDAP information collect is finished. >> $ACTUATOR_LOG
 fi
 
 echo `date "+%a %D %T"` Finished getting LDAP users and groups. >> $ACTUATOR_LOG
fi

echo Merging LDAP and roles information
cat $OUTPUT_FILE | sed -e 's/\//\\/g' >> $TEMPFILE1
if [ -f $LOCAL_OUTPUT_FILE ] ; then 
 cat $LOCAL_OUTPUT_FILE >> $TEMPFILE1
fi

rm -rf $TEMP_DIR
echo `date "+%a %D %T"` Processing finished. >> $ACTUATOR_LOG
return 0
